package shanghu

import (
	"duoduo/apis/common"
	"duoduo/apis/pdd"
	"duoduo/apis/shanghu/models"
	"duoduo/models/shanghu"
	"duoduo/tools"
	"duoduo/tools/app"
	"fmt"
	"github.com/gin-gonic/gin"
	"time"
)

// 注册用户
func OpenIdSet(c *gin.Context) {
	var inData models.OpenIdSetRequest
	var outData models.OpenIdSetResponse
	var sqlData shanghu.MerchantUser
	//var sql1 bizhi.User

	err := c.ShouldBindJSON(&inData)
	if err != nil {
		app.Error(c, 400, err, err.Error())
		return
	}

	url := fmt.Sprintf("https://api.weixin.qq.com/sns/jscode2session?appid=wxcf8e7dbe7d7e1ebd&secret=98195b9faacbd021e1724f9b1581cec2&js_code=%s&grant_type=authorization_code", inData.Code)

	val, err := pdd.DuoDuoGet(url)
	if err != nil {
		app.Error(c, 500, err, err.Error())
		return
	}

	err = tools.JsonUnmarshal(val, &outData)
	if err != nil {
		app.Error(c, 500, err, err.Error())
		return
	}
	if outData.Errcode != 0 {
		app.Error(c, outData.Errcode, err, outData.Errmsg)
		return
	}
	//查询数据
	sqlData.OpenID = outData.OpenId
	num := sqlData.GetNum()
	if num == 0 {
		//创建用户
		sqlData.CreatedAt = time.Now()
		sqlData.UpdatedAt = time.Now()
		//校验唯一性
		for _, v := range common.Example(6, 10) {
			sqlData.Code = v
			if sqlData.GetCodeNum() <= 0 {
				break
			}
		}

		_, err = sqlData.Create()
		if err != nil {
			app.Error(c, 500, err, err.Error())
			return
		}
	}

	app.OK(c, outData, app.Success)

}

// 注册用户
func ClientOpenIdSet(c *gin.Context) {
	var inData models.OpenIdSetRequest
	var outData models.OpenIdSetResponse
	var sqlData shanghu.MerchantClientUser

	err := c.ShouldBindJSON(&inData)
	if err != nil {
		app.Error(c, 400, err, err.Error())
		return
	}

	fmt.Println(inData)
	url := fmt.Sprintf("https://api.weixin.qq.com/sns/jscode2session?appid=wx8595c589dd736486&secret=668f87d2bc24199688e53ee8a88434b8&js_code=%s&grant_type=authorization_code", inData.Code)

	val, err := pdd.DuoDuoGet(url)
	if err != nil {
		app.Error(c, 500, err, err.Error())
		return
	}

	err = tools.JsonUnmarshal(val, &outData)
	if err != nil {
		app.Error(c, 500, err, err.Error())
		return
	}
	if outData.Errcode != 0 {
		app.Error(c, outData.Errcode, err, outData.Errmsg)
		return
	}
	//查询数据
	sqlData.ClientOpenID = outData.OpenId
	num := sqlData.GetNum()
	if num == 0 {
		//创建用户
		sqlData.CreatedAt = time.Now()
		sqlData.UpdatedAt = time.Now()
		//校验唯一性
		for _, v := range common.Example(6, 10) {
			sqlData.Code = v
			if sqlData.GetCodeNum() <= 0 {
				break
			}
		}

		_, err = sqlData.Create()
		if err != nil {
			app.Error(c, 500, err, err.Error())
			return
		}
	}

	app.OK(c, outData, app.Success)

}

func GetUserInfo(c *gin.Context) {
	var inData models.GetUserInfoRequest
	var sqlData shanghu.MerchantUser

	err := c.ShouldBindJSON(&inData)
	if err != nil {
		app.Error(c, 400, err, err.Error())
		return
	}

	//查询数据
	sqlData.OpenID = inData.OpenId
	userInfo, err := sqlData.GetUserInfo()
	if err != nil {
		app.Error(c, 500, err, err.Error())
		return
	}

	app.OK(c, userInfo, app.Success)

}

func ClientGetUserInfo(c *gin.Context) {
	var inData models.GetUserInfoRequest
	var sqlData shanghu.MerchantClientUser

	err := c.ShouldBindJSON(&inData)
	if err != nil {
		app.Error(c, 400, err, err.Error())
		return
	}

	//查询数据
	sqlData.ClientOpenID = inData.OpenId
	userInfo, err := sqlData.GetUserInfo()
	if err != nil {
		app.Error(c, 500, err, err.Error())
		return
	}

	app.OK(c, userInfo, app.Success)

}

func MerchantUserCode(c *gin.Context) {
	var inData models.MerchantUserCodeRequest
	var sqlData shanghu.MerchantUser
	var outData models.MerchantUserCodeResponse

	err := c.ShouldBindJSON(&inData)
	if err != nil {
		app.Error(c, 400, err, err.Error())
		return
	}

	//查询数据
	sqlData.Code = inData.Code
	userInfo, err := sqlData.GetUserInfoByCode()
	if err != nil {
		app.Error(c, 500, err, err.Error())
		return
	}
	outData.OpenId = userInfo.OpenID

	app.OK(c, outData, app.Success)

}
