package models

import (
	"github.com/shopspring/decimal"
)

type ClientTransRequest struct {
	OpenId    string `json:"open_id"`
	PageSize  int    `json:"page_size"`
	PageIndex int    `json:"page_index"`
	TypeList  int    `json:"type_list"` //1-未开始 2-进行中 3-已结束
}

type ClientTransReply struct {
	MerchantCardID    int64           `json:"merchant_card_id"`    // 商户卡id
	Amount            decimal.Decimal `json:"amount"`              // 交易金额
	PayTime           string          `json:"pay_time"`            // 支付时间
	MerchantCardName  string          `json:"merchant_card_name"`  // 商户卡名称
	MerchantCardTime  string          `json:"merchant_card_time"`  // 商户卡有效期
	TransId           int64           `json:"trans_id"`            // 交易id
	TotalCancelNumber int             `json:"total_cancel_number"` // 总核销次数
	CancelNumber      int             `json:"cancel_number"`       // 待核销次数
	CommAmount        decimal.Decimal `json:"comm_amount"`         //佣金
}

type VerificationCodeRequest struct {
	TransId int64 `json:"trans_id"`
}

type QRData struct {
	MerchantId   int64  `json:"merchant_id"`
	Key          string `json:"key"`
	ClientOpenId string `json:"client_open_id"`
}

type VerificationCodeReply struct {
	QR string `json:"qr"`
}

type MerchantTransRequest struct {
	MerchantOpenId string `json:"open_id"`
	PageSize       int    `json:"page_size"`
	PageIndex      int    `json:"page_index"`
}

type MerchantTransReply struct {
	MerchantCardID   int64           `json:"merchant_card_id"`   // 商户卡id
	TotalAmount      decimal.Decimal `json:"amount"`             // 交易金额
	MerchantAmount   decimal.Decimal `json:"merchant_amount"`    // 商家获得金额
	SaleAmount       decimal.Decimal `json:"sale_amount"`        //分销佣金
	PayTime          string          `json:"pay_time"`           // 支付时间
	MerchantCardName string          `json:"merchant_card_name"` // 商户卡名称
	MerchantCardTime string          `json:"merchant_card_time"` // 商户卡有效期
	TransId          int64           `json:"trans_id"`           // 交易id
	SaleId           string          `json:"sale_id"`            //销售ID
}

type ClientTransListRequest struct {
	ClientOpenId string `json:"open_id"`
	PageSize     int    `json:"page_size"`
	PageIndex    int    `json:"page_index"`
}

type ClientTransListReply struct {
	MerchantCardID   int64           `json:"merchant_card_id"`   // 商户卡id
	SaleAmount       decimal.Decimal `json:"sale_amount"`        // 分销佣金
	PayTime          string          `json:"pay_time"`           // 支付时间
	MerchantCardName string          `json:"merchant_card_name"` // 商户卡名称
	TransId          int64           `json:"trans_id"`           // 交易id
	UserId           string          `json:"user_id"`            // 用户ID
}
