package shanghu

import (
	orm "duoduo/database"
	"errors"
	"github.com/shopspring/decimal"
	"time"
)

type CashOut struct {
	ID               int64           `gorm:"column:id;type:bigint(20);primary_key" json:"id"`                        // Id
	OpenID           string          `gorm:"column:open_id;type:varchar(255)" json:"open_id"`                        // Openid
	AppID            string          `gorm:"column:app_id;type:varchar(255)" json:"app_id"`                          // Appid
	Status           int             `gorm:"column:status;type:int(11)" json:"status"`                               // 0-待提现 1-提现中 2-提现失败 3-钱已到账待账户扣减 99-提现成功
	FailRes          string          `gorm:"column:fail_res;type:varchar(255)" json:"fail_res"`                      // 失败原因
	PartnerTradeNo   string          `gorm:"column:partner_trade_no;type:varchar(50)" json:"partner_trade_no"`       // 商户卡提现id
	Amount           decimal.Decimal `gorm:"column:amount;type:decimal(10,2)" json:"amount"`                         // 金额
	CornTime         time.Time       `gorm:"column:corn_time;type:datetime;default:null" json:"corn_time"`           // 定时任务下次查询状态时间
	CreateBy         int64           `gorm:"column:create_by;type:bigint(20)" json:"create_by"`                      // 创建者
	UpdateBy         int64           `gorm:"column:update_by;type:bigint(20)" json:"update_by"`                      // 更新者
	CreatedAt        time.Time       `gorm:"column:created_at;type:datetime(3)" json:"created_at"`                   // 创建时间
	UpdatedAt        time.Time       `gorm:"column:updated_at;type:datetime(3)" json:"updated_at"`                   // 最后更新时间
	DeletedAt        time.Time       `gorm:"column:deleted_at;type:datetime(3);default:null" json:"deleted_at"`      // 删除时间
	Fee              decimal.Decimal `gorm:"column:fee;type:decimal(10,2)" json:"fee"`                               // 手续费
	WxPartnerTradeNo string          `gorm:"column:wx_partner_trade_no;type:varchar(50)" json:"wx_partner_trade_no"` // wx商户卡提现id
	AccountStatus    int32           `gorm:"column:account_status;default:0;comment:'分账状态 99-分账成功 2-分账失败 0-未分账 3-分账中'" json:"account_status"`
	AccountFailRes   string          `gorm:"column:account_fail_res;type:varchar(255);comment:分账失败原因" json:"account_fail_res"`
}

func (m *CashOut) TableName() string {
	return "cash_out"
}

func (m *CashOut) GetCashOutByStatusNum(status []int) int {
	var count int

	table := orm.ShMysql.Table(m.TableName())
	table = table.Where("open_id = ? and status in (?) and app_id = ? ", m.OpenID, status, m.AppID)
	table.Count(&count)

	return count

}

func (u *CashOut) Create() (CashOut, error) {
	var doc CashOut
	var err error

	doc = *u
	err = orm.ShMysql.Table(u.TableName()).Create(&doc).Error
	if err != nil {
		return doc, err
	}

	return doc, nil
}

func (m *CashOut) UpdateMerchantStatus() error {

	if err := orm.ShMysql.Table(m.TableName()).Model(&m).Where("id = ? ", m.ID).Updates(
		map[string]interface{}{
			"status":     m.Status,
			"fail_res":   m.FailRes,
			"updated_at": time.Now()}).Error; err != nil {
		return err
	}

	return nil
}

func (m *CashOut) UpdateCashOutWxBachNo() error {

	if err := orm.ShMysql.Table(m.TableName()).Model(&m).Where("partner_trade_no = ? ", m.PartnerTradeNo).Updates(
		map[string]interface{}{
			"wx_partner_trade_no": m.WxPartnerTradeNo,
			"updated_at":          time.Now()}).Error; err != nil {
		return err
	}

	return nil
}

func (m *CashOut) UpdateCashOut() error {

	if err := orm.ShMysql.Table(m.TableName()).Model(&m).Where("partner_trade_no = ? ", m.PartnerTradeNo).Updates(
		map[string]interface{}{
			"wx_partner_trade_no": m.WxPartnerTradeNo,
			"updated_at":          time.Now()}).Error; err != nil {
		return err
	}

	return nil
}

func (m *CashOut) UpdateCashOutStatus() error {

	if err := orm.ShMysql.Table(m.TableName()).Model(&m).Where("partner_trade_no = ? ", m.PartnerTradeNo).Updates(
		map[string]interface{}{
			"status":     m.Status,
			"updated_at": time.Now()}).Error; err != nil {
		return err
	}

	return nil
}

func (m *CashOut) UpdateCashOutStatusClose() error {

	if err := orm.ShMysql.Table(m.TableName()).Model(&m).Where("partner_trade_no = ? ", m.PartnerTradeNo).Updates(
		map[string]interface{}{
			"status":     m.Status,
			"fail_res":   m.FailRes,
			"updated_at": time.Now()}).Error; err != nil {
		return err
	}

	return nil
}

func (m *CashOut) GetCashOutByStatus() (CashOut, error) {
	var doc CashOut

	table := orm.ShMysql.Table(m.TableName())
	table = table.Where("status = ? ", m.Status)

	if err := table.Select("*").First(&doc).Error; err != nil {
		return doc, err
	}
	return doc, nil

}

func (m *CashOut) GetCashOutByAccount() (CashOut, error) {
	var doc CashOut

	table := orm.ShMysql.Table(m.TableName())
	table = table.Where("status = ? and account_status = ?", m.Status, m.AccountStatus)

	if err := table.Select("*").First(&doc).Error; err != nil {
		return doc, err
	}
	return doc, nil

}

func (m *CashOut) UpdateCashOutAccountStatus() error {

	reply := orm.ShMysql.Table(m.TableName()).Model(&m).Where("id = ? ", m.ID).Updates(
		map[string]interface{}{
			"account_status":   m.AccountStatus,
			"account_fail_res": m.AccountFailRes,
			"updated_at":       time.Now()})

	if reply.Error != nil {
		return reply.Error
	}

	if reply.RowsAffected <= 0 {
		return errors.New("更新记录0条")
	}

	return nil
}

// list 接口使用
func (m *CashOut) GetCashOutList(pageSize int, pageIndex int) ([]CashOut, int, error) {
	var doc []CashOut

	table := orm.ShMysql.Table(m.TableName())

	table = table.Where("open_id = ? and app_id = ? ", m.OpenID, m.AppID)
	var count int
	if err := table.Select("*").Order("id desc").Offset((pageIndex - 1) * pageSize).Limit(pageSize).Find(&doc).Error; err != nil {
		return nil, 0, err
	}
	table.Count(&count)
	return doc, count, nil
}
